-module(gaselect).
-export([generate_weights/4, roulette_wheel/2]).

%
% GA selection related code
%
% matt@cs.uoregon.edu
%

max(A,B) ->
  if
    (A > B) -> A;
    true -> B
  end.

make_weights(0,_,_,_) -> [];
make_weights(N,Cur,ScaleVal,MinVal) ->
  Cval = max(Cur*ScaleVal, MinVal),
  [Cval]++(make_weights(N-1,Cval, ScaleVal, MinVal)).  
  
normalize_weights(W) ->
  Wadjust = 1.0 / lists:foldl(fun(X,Sum)->X+Sum end, 0, W),
  lists:map(fun(X) -> X*Wadjust end, W).

generate_weights(N,Cur,ScaleVal,MinVal) ->
  normalize_weights(make_weights(N,Cur,ScaleVal,MinVal)).
  
find_bin(_,N,_,[]) -> N;
find_bin(Tot,N,Sample,[H|T]) ->
  if 
    (Sample>Tot) andalso (Sample =< (Tot+H)) -> N;
    true -> find_bin(Tot+H,N+1,Sample,T)
  end.
  
roulette_wheel(0,_) -> [];
roulette_wheel(N,W) -> 
  Sample = random:uniform(),
  [find_bin(0.0, 1, Sample, W)]++(roulette_wheel(N-1,W)).
  